@@ -1,25 +1,24 @@
 use v6;
 
-module File::Spec-0.0.1;
-
-# what follows is a really horrid hack, 
-# and a perverse exploitation of the 
-# current (unfinished) state of modules
-# in pugs. Please do not hold it against 
-# me :)
-
 if ($?OS eq 'MSWin32') {
-    eval 'require File::Spec::Win32';
+    require File::Spec::Win32;
 }
 else {
-    eval 'require File::Spec::Unix';
+    require File::Spec::Unix;
 }
 
+# The "require" lines above is deliberately
+# not part of the File::Spec package, because
+# we'd like to import on behalf of the caller.
+# Again, a horrible hack.
+
+module File::Spec-0.0.1;
+
 =kwid
 
 = NAME
 
-File::Spec - Perl6/Pugs Portable file handling
+File::Spec - Perl 6/Pugs Portable file handling
 
 = SYNOPOSIS
 
@@ -34,7 +33,7 @@ File::Spec - Perl6/Pugs Portable file handling
 
 = DESCRIPTION
 
-This is a very primative port of the perl5 File::Spec module. Since 
+This is a very primative port of the Perl 5 File::Spec module. Since 
 we currently do not have objects or fully functioning modules in 
 Pugs, this port does it's best to work with those limitations and 
 still produce a working version of File::Spec for use as we develop 
@@ -43,7 +42,7 @@ Pugs.
 = LIMITATIONS & CAVEATS
 
 Since we don't yet have object support, this module is more like the
-perl5 File::Spec::Functions module than the base File::Spec. Also the
+Perl 5 File::Spec::Functions module than the base File::Spec. Also the
 /hack/ to make it work automagically for platforms is really bad. I 
 hope that as Pugs matures this will change, but for now, it works :).
 
@@ -90,7 +89,7 @@ GHC and Pugs currently run on (yeah no VMS !!).
 
 = SEE ALSO
 
-The Perl5 version of File::Spec, although this version is more akin 
+The Perl 5 version of File::Spec, although this version is more akin 
 to File::Spec::Functions.
 
 = AUTHOR
@@ -101,7 +100,7 @@ Max Maischein <corion@cpan.org>
 
 = ACKNOWLEDGEMENTS
 
-This is a port of the perl5 File::Spec module which is currently 
+This is a port of the Perl 5 File::Spec module which is currently 
 maintained by Ken Williams <KWILLIAMS@cpan.org>, and is written
 by a number of people. Please see that module for more information.
 
